<?php
namespace OrganizationsManagement\Service;
use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Messages as Messages;
use OrganizationsManagement\Factory;
use Teknisa\Model\Teknisa\Repositories;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;

class Elemento
{
    /** @var Repositories\OmProduto **/
    protected $repositoryProduto;

    /** @var Repositories\OmProdutoOrg **/
    protected $repositoryProdutoOrg;

    /** @var Repositories\OmProdutoElemento **/
    protected $repositoryProdutoElemento;

    /** @var Repositories\OmProdutoOrgElemento **/
    protected $repositoryProdutoOrgElemento;

    /** @var Utilities **/
    protected $utilities;

    /** @var EntityManager **/
    protected $entityManager;

    /** @var TeknisaEnvironment **/
    protected $environment;

    /** @var Factory\Elemento **/
    protected $factoryElemento;

    /** @var Factory\ProdutoOrg **/
    protected $factoryProdutoOrg;

    public function __construct(EntityManager $entityManager, Utilities $utilities, TeknisaEnvironment $environment, Factory\Elemento $factoryElemento, Factory\ProdutoOrg $factoryProdutoOrg){
        $this->entityManager = $entityManager;
        $this->utilities = $utilities;
        $this->environment = $environment;
        $this->factoryElemento = $factoryElemento;
        $this->factoryProdutoOrg = $factoryProdutoOrg;
        $this->repositoryProduto = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProduto');
        $this->repositoryProdutoElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoElemento');
        $this->repositoryProdutoOrg = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrg');
        $this->repositoryProdutoOrgElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrgElemento');
    }

    
    public function save($productId, $urlJsonFile, $defaultElementIds, $elementIds) {
        $newElementsArray = array();
        $connection = $this->entityManager->getConnection();
        
        try {
            $connection->beginTransaction();
            $elementsMenu = $this->utilities->getElementsMenu($urlJsonFile, $productId);
            $response = $this->saveElements($elementsMenu, $defaultElementIds, $elementIds);
            $this->checkAndDeleteElementsUnused($productId, $response['NEW_ELEMENTS']);
            $connection->commit();
            return $response;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function saveElements($elementsMenu, $defaultElementIds = array(), $elementIds = array()) {
        $dateNow = new \DateTime();
        $orgId = $this->environment->getCurrentOrganizationId();
        $codOperador = $this->environment->getCurrentUserId();
        $newElements = array();
        $unsavedElements = array();
        foreach ($elementsMenu as $element) {
            if ($element['UNIQUE_ID']) {
                $hasElement = $this->repositoryProdutoElemento->findOneBy(array(
                    'uniqueId' => $element['UNIQUE_ID'],
                    'nrprodutoId' => $element['PROJECT_ID']
                    )
                );
                if($element['PARENT_ID']){
                    $parentId = $this->repositoryProdutoElemento->findOneBy(array(
                        'uniqueId' => $element['PARENT_ID'],
                        'nrprodutoId' => $element['PROJECT_ID']
                        )
                    )->getId();
                } else {
                    $parentId = null;
                }
                if($hasElement) {
                    if(in_array($element['UNIQUE_ID'], $elementIds) && in_array($element['UNIQUE_ID'], $defaultElementIds)) {
                        $defaultElement = 'S';
                    } else if(in_array($element['UNIQUE_ID'], $elementIds) && !in_array($element['UNIQUE_ID'], $defaultElementIds)) {
                        $defaultElement = 'N';
                    } else {
                        $defaultElement = $hasElement->getIdelementopadrao();
                    }
                } else {
                    $defaultElement = in_array($element['UNIQUE_ID'], $defaultElementIds) ? 'S' : 'N';
                }
                $savedElement = $this->factoryElemento->save($hasElement, $element['UNIQUE_ID'], $parentId, $element['PROJECT_ID'], 
                    $orgId, $element['LABEL'], $codOperador, $dateNow, $element['EX_PRODUCT_ID'], $element['NRORDEM'], $defaultElement);
                $this->entityManager->persist($savedElement);
                $this->entityManager->flush();

                $omProdutoOrgEspec = InstanceProvider::getRepository()->getOmProdutoElementoOrgEspec()->findBy(array('nrprodutoElementoId' => $savedElement->getId()));
                if (!empty($element['SPECIFIC_ORG']) || !empty($omProdutoOrgEspec)) {
                    $this->saveSpecificOrgElements($element, $savedElement->getId());
                }

                $newElements[] = $element['UNIQUE_ID'];
            } else {
                $unsavedElements[] = $element['LABEL'];
            }
        }
        return array('NEW_ELEMENTS' => $newElements, 'UNSAVED_ELEMENTS' => $unsavedElements);
    }

    private function saveSpecificOrgElements($element, $elementId) {
        $organizations = array();
        $dateNow       = new \DateTime();
        $codOperador   = $this->environment->getCurrentUserId();
        $currentOrgId  = $this->environment->getCurrentOrganizationId();

        $this->removeOmProdutoElementoOrgEspec($elementId);
        if(!empty($element['SPECIFIC_ORG'])) {
            foreach(explode(',', $element['SPECIFIC_ORG']) as $org) {
                $organizations[] = array('NRORG' => $org);
                $savedElement = $this->factoryElemento->saveElementOrgEspec($elementId, $org, $currentOrgId, $dateNow, $codOperador);
                $this->entityManager->persist($savedElement);
            }
        }
        $this->entityManager->flush();
        $this->removeOrgOmProdutoElementoOrgEspec($elementId);
        $this->saveOrgElements([$element], $organizations, $codOperador, $currentOrgId, true);
    }

    private function removeOrgOmProdutoElementoOrgEspec($elementId) {
        $productOrgElement = $this->repositoryProdutoOrgElemento->findBy(array('nrprodutoElementoId' => $elementId));
        foreach($productOrgElement as $orgElement) {
            InstanceProvider::getService()->getOrgElemento()->removeOmProdOrgElementoContratoByNrprodutoOrgElementoId($orgElement->getId());
            $this->entityManager->remove($orgElement);
        }
        $this->entityManager->flush();
    }

    private function removeOmProdutoElementoOrgEspec($elementId) {
        $omProdutoElementoOrgEspecRepository = InstanceProvider::getRepository()->getOmProdutoElementoOrgEspec();
        $omProdutoOrgEspec = $omProdutoElementoOrgEspecRepository->findBy(array('nrprodutoElementoId' => $elementId));
        foreach ($omProdutoOrgEspec as $element) {
            $this->entityManager->remove($element);
        }
        $this->entityManager->flush();
    }
    
    private function checkAndDeleteElementsUnused($productId, $newElementsArray) {
        $notInFilter = " AND (" . Utilities::fixBugWhereINQuery($newElementsArray, 'UNIQUE_ID', true, true) . ")";
        $elementsUnused = $this->repositoryProdutoElemento->getElementsToDeleteWithNotInFilter($productId, $notInFilter);
        foreach ($elementsUnused as $element) {
            $productOrgElementToDeleteArray = $this->repositoryProdutoOrgElemento->findBy(array('nrprodutoElementoId' => $element['ID']));
            if ($productOrgElementToDeleteArray){
                foreach($productOrgElementToDeleteArray as $productOrgElement){
                    InstanceProvider::getService()->getOrgElemento()->removeOmProdOrgElementoContratoByNrprodutoOrgElementoId($productOrgElement->getId());
                    $this->entityManager->remove($productOrgElement);
                }
            }
            $this->entityManager->flush();
            $productElement = $this->repositoryProdutoElemento->findOneBy(array(
                'uniqueId' => $element['UNIQUE_ID'],
                'nrprodutoId' => $element['PROJECT_ID']
                )
            );
            $this->removeOmProdutoElementoOrgEspec($productElement->getId());
            $this->entityManager->remove($productElement);
            $this->entityManager->flush();
        }
    }

    public function saveElementsInOrganizations($elements, $organizations){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
            try {
                $currentOrgId = $this->environment->getCurrentOrganizationId();
                $codOperador = $this->environment->getCurrentUserId();
                $this->saveOrgElements($elements, $organizations, $codOperador, $currentOrgId);
                $this->entityManager->flush();
                $connection->commit();
                return $elements;
            } catch (\Exception $e){
                $connection->rollBack();
                throw $e;
            }
    }

    public function saveOrgElements($elements, $organizations, $codOperador, $currentOrgId, $saveParents = false) {
        $dateNow = new \DateTime();
        foreach($organizations as $org){
            foreach($elements as $element){
                $elementId = $this->repositoryProdutoElemento->findOneBy(array(
                        'uniqueId' => $element['UNIQUE_ID'],
                        'nrprodutoId' => $element['PROJECT_ID']
                    )
                )->getId();
                $productOrg = $this->repositoryProdutoOrg->findOneBy(array(
                    'nrorgId'     => $org['NRORG'],
                    'nrprodutoId' => $element['PROJECT_ID']
                ));
                $productElement = $this->repositoryProdutoElemento->findOneBy(array(
                    'uniqueId'    => $element['UNIQUE_ID'],
                    'nrprodutoId' => $element['PROJECT_ID']
                ));
                $productElementOrg = $this->repositoryProdutoOrgElemento->findOneBy(array(
                    'nrprodutoOrgId'      => $productOrg ? $productOrg->getId() : 0,
                    'nrprodutoElementoId' => $productElement->getId()
                ));
                if(!$productElementOrg) {
                    if(!$productOrg) {
                        $productOrg = $this->factoryProdutoOrg->save(null, $org['NRORG'], $element['PROJECT_ID'], $codOperador, $currentOrgId, $dateNow);
                        $this->entityManager->persist($productOrg);
                        $this->entityManager->flush();
                    }
                    $savedElement = $this->factoryElemento->saveElementProductOrg($productOrg->getId(), $elementId, $currentOrgId, $dateNow, $codOperador);
                    $this->entityManager->persist($savedElement);
                    $this->entityManager->flush();
                    if($saveParents && $productElement->getParentId()) {
                        $this->setParentVisible($productElement->getParentId(), $productOrg->getId(), $currentOrgId, $dateNow, $codOperador);
                    }
                }
            }
        }
    }

    private function setParentVisible($elementId, $productOrgId, $currentOrgId, $dateNow, $codOperador) {
        $productElement = $this->repositoryProdutoElemento->findOneBy(array(
            'id' => $elementId
        ));
        $productElementOrg = $this->repositoryProdutoOrgElemento->findOneBy(array(
            'nrprodutoOrgId'      => $productOrgId,
            'nrprodutoElementoId' => $elementId
        ));
        if(empty($productElementOrg)) {
            $savedElement = $this->factoryElemento->saveElementProductOrg($productOrgId, $elementId, $currentOrgId, $dateNow, $codOperador);
            $this->entityManager->persist($savedElement);
            $this->entityManager->flush();
        }
        if(!empty($productElement->getParentId())) {
            $this->setParentVisible($productElement->getParentId(), $productOrgId, $currentOrgId, $dateNow, $codOperador);
        }
    }

    public function getMenuChanges($urlJsonFile, $productId){
        try {
            $newMenuElements = $this->utilities->getElementsMenu($urlJsonFile, $productId);
            if(!$newMenuElements){
                throw new \Exception(Messages::ERR_URL_UNDEFINED);
            }
            $currentMenu = $this->repositoryProdutoElemento->getElements($productId, "");
            $changedMenu = $movedElements = $currentElementsId = $newElementsId = array();
            
            foreach($currentMenu as $currentMenuElements){
                array_push( $currentElementsId , $currentMenuElements['UNIQUE_ID']);
            }
            foreach($newMenuElements as &$newElement){
                $newElement['STATUS'] = "I";
                array_push($newElementsId, $newElement['UNIQUE_ID']);
            }
            $intersectionIds = array_intersect($newElementsId, $currentElementsId);

            foreach ($intersectionIds as $same){
                $elementOnNewMenu = null;
                $elementOnCurrentMenu = null;
                foreach ($newMenuElements as $newElement){
                    if($newElement['UNIQUE_ID'] == $same){
                        $elementOnNewMenu = $newElement;
                    }
                }
                foreach ($currentMenu as $currentElement){
                    if($currentElement['UNIQUE_ID'] == $same){
                        $elementOnCurrentMenu = $currentElement;
                    }
                }
                
                if(!empty($elementOnCurrentMenu) && !empty($elementOnNewMenu['PARENT_ID']) && $elementOnCurrentMenu['PARENT_UNIQUE_ID'] != $elementOnNewMenu['PARENT_ID']) {
                    $elementOnNewMenu['STATUS'] = 'M';
                    $elementOnNewMenu['IDELEMENTOPADRAO'] = $elementOnCurrentMenu['IDELEMENTOPADRAO'];
                    array_push($movedElements, $elementOnNewMenu);
                }
            }

            foreach($currentElementsId as $currentId){
                if(!in_array($currentId, $newElementsId)){
                    foreach ($currentMenu as $removed){
                        if($removed['UNIQUE_ID'] == $currentId){
                            $removed['STATUS'] = 'R';
                            array_push($newMenuElements, $removed);
                        }
                    }
                }
            }

            foreach($newElementsId as $newId){
                if(!in_array($newId, $currentElementsId)){
                    foreach ($newMenuElements as &$added){
                        if($added["UNIQUE_ID"] == $newId){
                            $added['STATUS'] = 'A';
                        }
                    }
                }
            }

            foreach($newMenuElements as &$newElement){
                foreach($currentMenu as $currentElement) {
                    if($newElement['UNIQUE_ID'] == $currentElement['UNIQUE_ID'] && 
                       ($newElement['LABEL'] != $currentElement['LABEL'] || $newElement['SPECIFIC_ORG'] != $currentElement['SPECIFIC_ORG'])) {
                        $newElement['IDELEMENTOPADRAO'] = $currentElement['IDELEMENTOPADRAO'];
                        $newElement['STATUS'] = "E";
                    }
                }
            }

            foreach($newMenuElements as &$element){
                if(isset($element['_parent']['UNIQUE_ID'])){
                    $element['PARENT_ID'] = $element['_parent']['UNIQUE_ID'];
                }
            }

            $newMenuElements = array_merge($movedElements, $newMenuElements);

            foreach($newMenuElements as &$element){
                $element['PATH'] = null;
                $isMoved = false;
                if($element['PARENT_ID']){
                    $path = $this->getParent($newMenuElements, $element['PARENT_ID'], array(), $isMoved);
                    if($isMoved){
                        $element['STATUS'] = 'M';
                    }
                    if(is_array($path)){
                        $completePath = implode(" -> ", $path);
                        $element['PATH'] = $completePath;
                    } else{
                        $element['PATH'] = $path;
                    }
                }
                if($element['STATUS'] != 'I') {
                    if(!empty($element['SPECIFIC_ORG'])) {
                        $element['SPECIFIC_ORG_LABEL'] = InstanceProvider::getRepository()->getOrganization()->getOrganizationNamesConcatenated($element['SPECIFIC_ORG']);
                    }
                    array_push($changedMenu, $element);
                }
            }

            return $changedMenu;
        } catch (\Exception $e){
            throw $e;
        }
    }

    public function getParent($newMenuElements, $parentId, $path = array(), &$isMoved = false){
        foreach($newMenuElements as $element){
            if($element['UNIQUE_ID'] == $parentId){
                if($element['PARENT_ID']) {
                    if($element['STATUS'] == 'M'){
                        $isMoved = true;
                    }
                    array_unshift($path, $element['LABEL']);
                    return $this->getParent($newMenuElements, $element['PARENT_ID'], $path);
                } else if($element['TYPE'] == 'G' && sizeof($path) == 0){
                    return $element['LABEL'];
                } else{
                    array_unshift($path, $element['LABEL']);
                    return $path;
                }
            }
        }
    }
    
    public function getElements($productId, $filters, $devOpsId = null) {
        $conditions = Utilities::createQueryFiltered($filters);
        if($devOpsId) {
            $product   = $this->repositoryProduto->findOneBy(array('nrexternalProductId' => $devOpsId));
            $productId = $product->getId();
        }

        $elementsToSort = $this->repositoryProdutoElemento->getElements($productId, $conditions);
        if ($elementsToSort){
            return Utilities::generateMenuElementsTreeGrid($elementsToSort);
        } else {
            return array();  
        }
    }

    public function saveDefaultElement($id) {
        $productElement = $this->repositoryProdutoElemento->findOneBy(array('id' => $id));
        $default = $productElement->getIdelementopadrao() == 'S' ? 'N' : 'S';
        $productElement->setIdelementopadrao($default);
        $this->entityManager->persist($productElement);
        $this->entityManager->flush();
    }
}