<?php
namespace OrganizationsManagement\Service;
use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\Messages as Messages;
use OrganizationsManagement\Factory;
use Teknisa\Model\Teknisa\Repositories;
use Teknisa\Model\Teknisa\Entities;
use Teknisa\Model\Services\Entities\FctContrato;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;

class OrgElemento
{
    /** @var Repositories\OmProduto **/
    protected $repositoryProduto;
    
    /** @var Repositories\OmProdutoElemento **/
    protected $repositoryProdutoElemento;
    
    /** @var Repositories\OmProdutoOrgElemento **/
    protected $repositoryProdutoOrgElemento;
    
    /** @var Repositories\OmProdutoOrg **/
    protected $repositoryProdutoOrg;

    /** @var Utilities **/
    protected $utilities;

    /** @var EntityManager **/
    protected $entityManager;

    /** @var TeknisaEnvironment **/
    protected $environment;

    /** @var Factory\Elemento **/
    protected $factoryElemento;
    
    /** @var Factory\OrgElemento **/
    protected $factoryOrgElemento;

    public function __construct(EntityManager $entityManager, Utilities $utilities, TeknisaEnvironment $environment, Factory\Elemento $factoryElemento, Factory\OrgElemento $factoryOrgElemento){
        $this->entityManager = $entityManager;
        $this->utilities = $utilities;
        $this->environment = $environment;
        $this->factoryElemento = $factoryElemento;
        $this->factoryOrgElemento = $factoryOrgElemento;
        $this->repositoryProduto = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProduto');
        $this->repositoryProdutoElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoElemento');
        $this->repositoryProdutoOrgElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrgElemento');
        $this->repositoryProdutoOrg = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrg');
    }
    
    public function save($elementId, $elementUniqueId, $productIdOrg, $productId, $parentId, $visibleOnly, $expirationDate, $permanentElements, $dsobservacao) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $codOperador = $this->environment->getCurrentUserId();
            $currentOrgId = $this->environment->getCurrentOrganizationId();
            $this->orgElementSave($currentOrgId, $codOperador, $elementId, $elementUniqueId, $productIdOrg, $productId,
                                  $parentId, $visibleOnly, $expirationDate, $permanentElements, false, $dsobservacao);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function updateElementsObservation($elementId, $productIdOrg, $dsobservacao, $dtvalidade) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $codOperador = $this->environment->getCurrentUserId();
            $this->orgElementUpdateObservation($codOperador, $elementId, $productIdOrg, $dsobservacao, $dtvalidade);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function updateElementsObservationInArray($elementId, $elementProductOrg, $dsobservacao, $dtvalidade) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $codOperador = $this->environment->getCurrentUserId();
            $this->orgElementUpdateObservationInArray($codOperador, $elementId, $elementProductOrg, $dsobservacao, $dtvalidade);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function relacionarElementsContracts($elements, $contratos, $dtValidade, $dsObservacao) {
        foreach ($elements as $element) {
            $expirationDate = \DateTime::createFromFormat('d/m/Y H:i:s', $dtValidade);
            $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $element['PRODUCT_ORG'], 'nrprodutoElementoId' => $element['ID']));
            if ($orgElement === null) {
                $this->save($element['ID'], $element['UNIQUE_ID'], $element['PRODUCT_ORG'], $element['PROJECT_ID'], $element['PARENT_ID'], false, $expirationDate, 'N', $dsObservacao);
                $this->saveRelacionamentoContrato($contratos, $element, null, $expirationDate, $dsObservacao);
            } else {
                $this->saveRelacionamentoContrato($contratos, $element, $orgElement, $expirationDate, $dsObservacao);
            }
        }
    }

    public function saveRelacionamentoContrato($contratos, $element, $orgElement, $expirationDate, $dsObservacao) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $dateNow = new \DateTime();
        $codOperador = $this->environment->getCurrentUserId();
        $currentOrgId = $this->environment->getCurrentOrganizationId();
        try {
            if($element['IS_RELATED_TO_CONTRACT'] === 'N' && $element['IDELEMENTOPADRAO'] === 'N') {
                foreach ($contratos as $contrato) {
                    if ($orgElement !== null) {
                        $produtoOrgElementoContratoFactory = InstanceProvider::getFactory()->getProdutoOrgElementoContrato();
                        $produtoOrgElementoContrato = $produtoOrgElementoContratoFactory->save($element['ID_PRODUTO_ORG_ELEMENTO'], $contrato, $currentOrgId,  $dateNow, $codOperador, 'S', 'S');
                        $this->entityManager->persist($produtoOrgElementoContrato);
                        $orgElement->setDtvalidade($expirationDate);
                        $orgElement->setDsobservacao($dsObservacao);
                        $this->entityManager->persist($orgElement);
                    } else {
                        $newOrgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $element['PRODUCT_ORG'], 'nrprodutoElementoId' => $element['ID']));
                        $produtoOrgElementoContratoFactory = InstanceProvider::getFactory()->getProdutoOrgElementoContrato();
                        $produtoOrgElementoContrato = $produtoOrgElementoContratoFactory->save($newOrgElement->getId(), $contrato, $currentOrgId,  $dateNow, $codOperador, 'S', 'S');
                        $this->entityManager->persist($produtoOrgElementoContrato);
                    }
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function orgElementSave($currentOrgId, $codOperador, $elementId, $elementUniqueId, $productIdOrg, $productId,
                                   $parentId, $visibleOnly, $expirationDate, $permanentElements, $isExtendContract = false, $dsobservacao = null) {
        if($permanentElements == 'S') {
            $expirationDate = null;
        } else if($expirationDate) {
            try {
                $expirationDate = \DateTime::createFromFormat('d/m/Y', $expirationDate);
            } catch(\Exception $e) {
                $expirationDate = null;
            }
        }
        $dateNow = new \DateTime();
        /** @var Entities\OmProdutoOrgElemento $orgElement */
        $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementId));
        if ($orgElement && $isExtendContract) {
            if(empty($expirationDate)) {
                throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_EXPIRATION_DATE_INVALID);
            }
            $orgElement->setDtvalidade($expirationDate);
            $orgElement->setDsobservacao($dsobservacao);
            $this->entityManager->persist($orgElement);
            $this->entityManager->flush();
        } else if ($orgElement && !$visibleOnly) { 
            $this->setChildrenElementInvisible($productIdOrg, $elementId, $parentId);
        } else {
            if(!($orgElement && $visibleOnly)) {
                $savedElement = $this->factoryOrgElemento->save($productIdOrg, $elementId, $currentOrgId, $dateNow, $codOperador, $expirationDate, 'S', $dsobservacao);
                $this->entityManager->persist($savedElement);
                $this->entityManager->flush();
            }
            if ($parentId){
                $this->setParentElementVisible($productIdOrg, $currentOrgId, $codOperador, $dateNow, $parentId, $expirationDate, $dsobservacao);
            }
            $this->setChildrenElementVisible($productIdOrg, $currentOrgId, $codOperador, $dateNow, $elementUniqueId, $productId, $expirationDate, $dsobservacao);        
        }
        $productOrgElement = $this->repositoryProdutoOrgElemento->getProdutoOrgElementsFromProduto($productIdOrg);
        if(empty($productOrgElement)) {
            $this->removeProductWithoutElements($productIdOrg);
        }
    }

    public function removeProductWithoutElements($id){
        $omProdutoOrgEntity = $this->repositoryProdutoOrg->findOneBy(array('id' => $id));
        $orgVersion =InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findBy(array('nrprodutoOrgId' => $id));
        foreach ($orgVersion as $versions) {
            $this->entityManager->remove($versions);
        }
        $this->entityManager->flush();
        $this->entityManager->remove($omProdutoOrgEntity);
        $this->entityManager->flush();
    }

    public function orgElementSaveIntegration($currentOrgId, $codOperador, $elementId, $productIdOrg, $expirationDate, $isExtendContract, $contractCode, $isUpdate, $relatedElement = false) {
        /** @var FctContrato $fctContrato */
        $fctContrato = InstanceProvider::getRepository()->getFctContrato()->findOneBy(array('nrseqcontrato' => $contractCode));
        $contract    = $fctContrato ? $fctContrato->getNrcontratofct() : $contractCode;
        $produtoOrgElementoContratoFactory = InstanceProvider::getFactory()->getProdutoOrgElementoContrato();
        try {
            $expirationDate = \DateTime::createFromFormat('d/m/Y', $expirationDate);
            $daysToAdd = Utilities::getContractDaysToAddParameter();
            $expirationDate->modify("+$daysToAdd day");
        } catch(\Exception $e) {
            $expirationDate = null;
        }
        $dateNow = new \DateTime();
        $formatedDate = $dateNow->format('d/m/Y');
        /** @var Entities\OmProdutoOrgElemento $orgElement */
        $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementId));
        if ($orgElement && $isUpdate) {
            if($isExtendContract) {
                if (empty($expirationDate)) {
                    throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_EXPIRATION_DATE_INVALID);
                }
                $dsobservacao = $orgElement->getDsobservacao() . "
- ($formatedDate) Elemento atualizado através da alteração do período de vigência do contrato: $contract";
                $orgElement->setDtvalidade($expirationDate);
            } else if($relatedElement) {
                $dsobservacao = $orgElement->getDsobservacao() . "
- ($formatedDate) Elemento relacionado através do contrato:  $contract";
            } else {
                $dsobservacao = $orgElement->getDsobservacao() . "
- ($formatedDate) Elemento atualizado através da renegociação do contrato: $contract";
            }
            /** @var Entities\OmProdutoOrgElementoContrato $omProdutoOrgElementoContrato */
            $omProdutoOrgElementoContrato = InstanceProvider::getRepository()->getOmProdutoOrgElementoContrato()
                                                                             ->findBy(array(
                                                                                        'nrprodutoOrgElementoId' => $orgElement->getId(),
                                                                                        'nrseqcontrato' => $contractCode
                                                                                    ));
            if(empty($omProdutoOrgElementoContrato)) {
                $produtoOrgElementoContrato = $produtoOrgElementoContratoFactory->save($orgElement->getId(), $contractCode, $currentOrgId,  $dateNow, $codOperador);
                $this->entityManager->persist($produtoOrgElementoContrato);
            }
            $dsobservacao = substr($dsobservacao,-4000);
            
            $orgElement->setDsobservacao($dsobservacao);
            $orgElement->setDtultatu($dateNow);
            $orgElement->setCdoperultatu($codOperador);
            $this->entityManager->persist($orgElement);
          /*  $this->factoryOrgElemento->updateElementosManuais($contractCode, $dsobservacao, $codOperador, $expirationDate); */
            $this->entityManager->flush();
        } else if(!$orgElement) {
            if($relatedElement) {
                $dsobservacao = "- ($formatedDate) Elemento relacionado. Elemento cadastrado através do contrato:  $contract";
            } else {
                $dsobservacao = "- ($formatedDate) Elemento cadastrado através do contrato: $contract";
            }
            $savedElement = $this->factoryOrgElemento->save($productIdOrg, $elementId, $currentOrgId, $dateNow, $codOperador, $expirationDate, 'S', $dsobservacao);
            $produtoOrgElementoContrato = $produtoOrgElementoContratoFactory->save($savedElement->getId(), $contractCode, $currentOrgId,  $dateNow, $codOperador);
            $this->entityManager->persist($savedElement);
            $this->entityManager->flush();
            $this->entityManager->persist($produtoOrgElementoContrato);
            $this->entityManager->flush();
        }
    }

    public function orgElementUpdateObservation($codOperador, $elementId, $productIdOrg, $dsobservacao, $dtvalidade = null) {
        $dateNow = new \DateTime();
        /** @var Entities\OmProdutoOrgElemento $orgElement */
        $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementId));
        $orgElement->setCdoperultatu($codOperador);
        $orgElement->setDtultatu($dateNow);
        $orgElement->setDsobservacao($dsobservacao);
        if($dtvalidade) {
             $dtvalidadeDateTime = \DateTime::createFromFormat('d/m/Y', $dtvalidade);
             $orgElement->setDtvalidade($dtvalidadeDateTime);
        }
        $this->entityManager->persist($orgElement);
        $this->entityManager->flush();
    }
    
    public function orgElementUpdateObservationInArray($codOperador, $elementIds, $productIdOrg, $dsobservacao, $dtvalidade = null) {
        $dateNow = new \DateTime();
        /** @var Entities\OmProdutoOrgElemento $orgElement */
        foreach($elementIds as $elementId){
            $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementId));
            if($orgElement) {
                $orgElement->setCdoperultatu($codOperador);
                $orgElement->setDtultatu($dateNow);
                if($dsobservacao) {
                    $orgElement->setDsobservacao($dsobservacao);
                }
                if($dtvalidade) {
                    $dtvalidadeDateTime = \DateTime::createFromFormat('d/m/Y', $dtvalidade);
                    $orgElement->setDtvalidade($dtvalidadeDateTime);
                } else {
                    $orgElement->setDtvalidade(null);
                }
                $this->entityManager->persist($orgElement);
            } else {
                 throw new \Exception(Messages::ERR_INACTIVE_ELEMENT);
            }
        }
        $this->entityManager->flush();
    }
    
    private function setParentElementVisible($productIdOrg, $currentOrgId, $codOperador, $dateNow, $elementId, $expirationDate, $dsobservacao){
        $element = $this->repositoryProdutoElemento->findOneBy(array('id' => $elementId));
        $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementId));
        if($orgElement === null) {
            $savedElement = $this->factoryOrgElemento->save($productIdOrg, $elementId, $currentOrgId, $dateNow, $codOperador, $expirationDate, 'S', $dsobservacao);
            $this->entityManager->persist($savedElement);
            $parentId = $element->getParentId();
            if ($parentId){
                return $this->setParentElementVisible($productIdOrg, $currentOrgId, $codOperador, $dateNow, $parentId, $expirationDate, $dsobservacao);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
    
    private function setChildrenElementVisible($productIdOrg, $currentOrgId, $codOperador, $dateNow, $elementUniqueId, $productId, $expirationDate, $dsobservacao) {
        $elementParent = $this->repositoryProdutoElemento->findOneBy(array('uniqueId' => $elementUniqueId, 'nrprodutoId' => $productId));
        if ($elementParent) {
            $elementChildren = $this->repositoryProdutoElemento->findBy(array('parentId' => $elementParent->getId(), 'nrprodutoId' => $elementParent->getNrprodutoId()));
            if ($elementChildren) {
                foreach ($elementChildren as $elementChild){
                    $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementChild->getId()));
                    if($orgElement === null) {
                        $savedElement = $this->factoryOrgElemento->save($productIdOrg, $elementChild->getId(), $currentOrgId, $dateNow, $codOperador, $expirationDate, 'S', $dsobservacao);
                        $this->entityManager->persist($savedElement);
                        $this->setChildrenElementVisible($productIdOrg, $currentOrgId, $codOperador, $dateNow, $elementChild->getUniqueId(), $productId, $expirationDate, $dsobservacao);
                    }
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
    
    public function setChildrenElementInvisible($productIdOrg, $elementId, $parentId){
        $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementId));
        if ($orgElement) {
            $productOrg = $this->repositoryProdutoOrg->findOneBy(array('id' => $productIdOrg));
            $omProdutoElementoOrgEspec = InstanceProvider::getRepository()->getOmProdutoElementoOrgEspec()->findOneBy(
                array('nrprodutoElementoId' => $elementId, 'nrorgId' => $productOrg->getNrorgId())
            );
            if(!$omProdutoElementoOrgEspec) {
                $elementChildren = $this->repositoryProdutoElemento->findBy(array('parentId' => $elementId));
                if ($elementChildren) {
                    foreach ($elementChildren as $elementChild) {
                        $this->setChildrenElementInvisible($productIdOrg, $elementChild->getId(), $elementChild->getParentId());
                    }
                }

                $this->removeOmProdOrgElementoContratoByNrprodutoOrgElementoId($orgElement->getId());

                $this->entityManager->remove($orgElement);
                $this->entityManager->flush();
                if ($parentId) {
                    $this->setParentElementInvisible($productIdOrg, $parentId);
                }
            }
        }
    }

    public function removeOmProdOrgElementoContratoByNrprodutoOrgElementoId($orgElementId)
    {
        $omProdOrgElementoContratoRepository = InstanceProvider::getRepository()->getOmProdutoOrgElementoContrato();
        $omProdOrgElementoContratos = $omProdOrgElementoContratoRepository->findBy(array('nrprodutoOrgElementoId' => $orgElementId));
        foreach ($omProdOrgElementoContratos as $elementContract) {
            $this->entityManager->remove($elementContract);
        }
        $this->entityManager->flush();
    }
    
    private function setParentElementInvisible($productIdOrg, $elementId){
        $element = $this->repositoryProdutoElemento->findOneBy(array('id' => $elementId));
        $elementParent = $this->repositoryProdutoElemento->findOneBy(array('id' => $element->getParentId()));
        $elementChildren = $this->repositoryProdutoElemento->findBy(array('parentId' => $elementId));
        $orgElement = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $elementId));
        if ($elementChildren) {
            foreach ($elementChildren as $elementChild){
                $arrayIdElementChildren[] = $elementChild->getId();
            }
            $orgElementChildren = $this->repositoryProdutoOrgElemento->findOneBy(array('nrprodutoOrgId' => $productIdOrg, 'nrprodutoElementoId' => $arrayIdElementChildren));
            if (!$orgElementChildren && $orgElement) {
                $this->removeOmProdOrgElementoContratoByNrprodutoOrgElementoId($orgElement->getId());
                $this->entityManager->remove($orgElement);
                $this->entityManager->flush();
                if ($elementParent) {
                    $this->setParentElementInvisible($productIdOrg, $elementParent->getId());
                }
            }
        }
    }
    
    public function getOrgElements($productId, $productIdOrg) {
        $elementsToSort = $this->repositoryProdutoOrgElemento->getOrgElements($productId, $productIdOrg);
        if ($elementsToSort) {
            return Utilities::generateMenuElementsTreeGrid($elementsToSort);
        } else {
            return array();
        }
    }    
    
    public function getChildrenOrgElements($id){
        $children=$this->repositoryProdutoOrgElemento->getChildrenOrgElements($id);
        return Utilities::getArrayFromAssociativeArray($children);
    }

    public function findContratoCliente($nrprodutoID, $orgID){
        return $this->repositoryProdutoOrgElemento->findContratoCliente($orgID, $nrprodutoID);
    }
}