function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise, MessageService) {
    var self = this;
    var loginUserId, loginUserName, loginIsSupOper, loginSupOperId;
    window.tekLoginProjectId = angular.copy(window.projectId);

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    var factoryPageViewNoticeWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getPageViewNotice(), templateManager.container);
    };

    var updateLanguage = function () {
        var langParam = Util.getUrlParameter('lang');
        var language = langParam || localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);
    };

    var updateToken = function () {
        var authToken = Util.getUrlParameter('authToken');
        if (authToken) {
            localStorage.setItem('LOGIN_TOKEN', authToken);
        }
        return authToken;
    };

    var updateHashAndKeepConn = function () {
        var hash = Util.getUrlParameter('hash');
        if (hash) {
            localStorage.setItem('LOGIN_HASH', hash);
        }
        var keepConn = Util.getUrlParameter('keepConn');
        if (keepConn) {
            localStorage.setItem('LOGIN_KEEP_CONNECTED', keepConn);
        }
    };

    var updateSeparator = function() {
        var separator = Util.getUrlParameter('sep') || localStorage.getItem('LOGIN_SEPARATOR');
        if(separator) {
            localStorage.setItem('LOGIN_SEPARATOR', separator);
            if($.currency.i18n[window.lang]) {
                $.currency.i18n[window.lang].thousands = separator[0];
                $.currency.i18n[window.lang].inputThousands = separator[0];
                $.currency.i18n[window.lang].decimal = separator[1];
                $.currency.i18n[window.lang].inputDecimal = separator[1];
            }
        }
    };

    this.authenticate = function(forceAuthenticate) {
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        updateLanguage();
        updateSeparator();
        updateHashAndKeepConn();

        var defer = ZHPromise.defer();
        forceAuthenticate = forceAuthenticate === true;

        if(window.location.hash.replace('#/', '') != LibUtilities.authenticationContainer || forceAuthenticate) {
            var setAclEventId;
            var user = localStorage.getItem('LOGIN_USER');
            var useRedirect = localStorage.getItem('LOGIN_USE_REDIRECT');
            var authToken = updateToken();

            if((!user && !authToken) || useRedirect == 'S') {
                LibUtilities.redirectToLogin();
                defer.resolve({});
            } else {
                var setAclFunction = function () {
                    setAcl(defer.promise, setAclEventId);
                };
                setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

                ScreenService.showLoader();
                LibLoginService.startSession(user).then(function (response) {
                    ScreenService.hideLoader();
                    startSessionResponse(response, defer);
                }).catch(function (error) {
                    ScreenService.hideLoader();
                    //Expired or Invalid Token
                    if(error.data && (error.data.errorCode == 101 || error.data.errorCode == 100)) {
                        self.logout();
                    } else if(error.data.errorCode == 20){
                        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
                        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
                            type: 'INFORMATION',
                            actions: [ {
                                label: ScreenService.i18n("Ok"),
                                code: function(){
                                    self.logout()
                                }
                            } ]
                        });
                    } else{
                        defer.reject(error);
                        LibUtilities.showErrorMessage(error);
                    }
                });
            }
        }

        return defer.promise;
    };

    var startSessionResponse = function(response, defer) {
        if (response.isError) {
            defer.reject(response.message);
        } else {
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                if(response.LICENSE_INFO) {
                    localStorage.setItem('LICENSE_INFO', response.LICENSE_INFO);
                }
                if(response.LOGIN_USER && response.LOGIN_TOKEN) {
                    localStorage.setItem('LOGIN_USER', response.LOGIN_USER);
                    localStorage.setItem('LOGIN_TOKEN', response.LOGIN_TOKEN);
                }
                
                localStorage.setItem('IS_SUPPORT_OPERATOR', response.IS_SUPPORT_OPERATOR);
                localStorage.setItem('LOGIN_USER_ID', response.USER_ID);
                localStorage.setItem('LOGIN_PHP_VERSION_ID', response.PHP_VERSION_ID);
                localStorage.setItem('ACL_ADMINISTRATOR', response.ACL_ADMINISTRATOR);
                localStorage.setItem('ACCESS_PROFILE', response.ACCESS_PROFILE);
                localStorage.setItem('LOGIN_CUSTOMIZATION_URL', response.CUSTOMIZATION_URL);
                localStorage.setItem('NOTICES', JSON.stringify(response.NOTICES));
                localStorage.setItem('TIMEOUTNOTICES', JSON.stringify(response.TIMEOUTNOTICES));

                if(response.IS_SUPPORT_OPERATOR) {
                    loginIsSupOper = response.IS_SUPPORT_OPERATOR;
                    loginSupOperId = response.SUPPORT_OPERATOR_ID;
                }

                loginUserId   = response.USER_ID;
                loginUserName = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    MessageService.showMessageByCode('lib#ALE_MIGRATION_IS_PENDENT', null, { type: 'INFORMATION' });
                }
                
                if(response.VISIBLE_ELEMENTS) {
                    localStorage.setItem('LOGIN_VISIBLE_ELEMENTS', JSON.stringify(response.VISIBLE_ELEMENTS));
                }

                if(response.ACL_ADMINISTRATOR) {                    
                    LibUtilities.addTranslateButton();
                }

                defer.resolve(response);
            }
        }
    };

    this.pageViewNoticeActionEventCancel = function () {
        ScreenService.closePopup(true);
    };

    this.openPopupNotices = function() {
        var route 	  = '/modules/login/pageViewNotice';
        var token 	  = localStorage.getItem('LOGIN_TOKEN');
        var hash 	  = localStorage.getItem('LOGIN_HASH') || '0';
        var user      = localStorage.getItem('LOGIN_USER');
        var languange = localStorage.getItem('LOGIN_LANGUAGE');
        
        // Clear angular session timeout to use only next session timeout.
        clearTimeout(LibUtilities.expiredSessionMethod);
        
        var url = window.nextUrl + route + "?header=0&menu=0" +
                                      "&token=" + token +
                                      "&hash=" + hash +
                                      "&user=" + user +
                                      "&languange=" + languange +
                                      "&productId=" + window.projectId +
                                      "&isZhAngular=1";     
        var pageViewNoticeWidget = factoryPageViewNoticeWidget();
        ScreenService.openPopup(pageViewNoticeWidget).then(function () {
            Util.doAsync(function () {
                var element = $('.zh-PAGEVIEWNOTICE').parent();
                if(element[0] && element[0].style) {
                    element[0].style.padding = "0px";
                }
                var popupElement = element.parent().parent().children();
                if(popupElement[0]) {
                    if(element[0] && element[0].style && popupElement[0].offsetHeight) {
                        element[0].style.height = (parseInt(element[0].style.height) + popupElement[0].offsetHeight) + "px";
                        element[0].style['max-height'] = element[0].style.height;
                    }
                    popupElement[0].remove();
                }
                var bodyHeight = element.height() -  $('.zh-footer-center').parent().height();
                var heightPDF = String(bodyHeight + "px");
                var htmlPDF = "<iframe id='pageViewNotice' src = "+ url + "#page=1&zoom=109' width='100%' height='100%' type='application/pdf' style='height:" + heightPDF + "; margin-bottom: 40px'></iframe>";
                var fieldSelector = $("#PDF");
                divField = fieldSelector.parent()[0];
                fieldSelector.parent()[0].innerHTML = htmlPDF;
                pageViewNoticeWidget.edit();
                templateManager.updateTemplate();
            }, 2000);
        });
    }

    this.logout = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('ACL_ADMINISTRATOR');
            localStorage.removeItem('ACCESS_PROFILE');
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LICENSE_INFO');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            localStorage.removeItem('IS_SUPPORT_OPERATOR');
            localStorage.removeItem('LOGIN_USER_ID');
            localStorage.removeItem('LOGIN_PHP_VERSION_ID');
            localStorage.removeItem('LIB_FAVORITES_DATA');
            localStorage.removeItem('NOTICES');
            localStorage.removeItem('TIMEOUTNOTICES');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };
 
    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    eventAggregator.subscribe("LibInvalidToken", function (errorCode) {
        //Expired Token
        if(errorCode == 101) {
            MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION', null, {
                type: 'INFORMATION',
                actions: [ {
                    label: ScreenService.i18n("Ok"),
                    code: self.logout
                } ]
            });
        }
        //Invalid Token
        else if(errorCode == 100) {
            self.logout()
        }
    });

    eventAggregator.subscribe("LibInvalidTokenSupportOperator", function (errorCode) {
        //Expired Token Support Operator
        MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION_SUPPORT_OPERATOR', null, {
            type: 'INFORMATION',
            actions: [{
                label: ScreenService.i18n("Ok"),
                code: self.logout
            }]
        });
    });

    eventAggregator.subscribe("LibConcurrentAccess", function () {
        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    eventAggregator.subscribe("LibAccessBlocked", function (errorCode) {
        MessageService.showCustomDialogByCode('lib#ACCESS_BLOCKED_BY_TIME_OR_DAY', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var message = afterLogin ? 'ERR_EXPIRED_SESSION_LOGIN' : 'ERR_ATTEMPT_END';
        MessageService.getMessageByCode('lib#' + message).then(function(loadingMessage) {
            ScreenService.changeLoadingMessage(ScreenService.i18n(loadingMessage));
            ScreenService.showLoader();
            Util.doAsync(self.logout, 5000);
        });
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password = expiredSessionWidget.currentRow.PASSWORD;
            var userId   = loginSupOperId || loginUserId;
            ScreenService.showLoader();
            LibLoginService.validateUser(userId, password, loginIsSupOper)
                .then(function() {
                    // Requisição getModulesVersion chamada para atualizar o tempo de sessão local (produto ou módulo)
                    LibLoginService.getModulesVersion();
                    ScreenService.hideLoader();
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function(error) {
                    ScreenService.hideLoader();
                    if(error.data && error.data.errorCode && error.data.errorCode == 20) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibConcurrentAccess');
                    } else if(error.data && error.data.errorCode && error.data.errorCode == 102) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibInvalidTokenSupportOperator');
                    } else if(error.data && error.data.errorCode && (error.data.errorCode == 21 || error.data.errorCode == 22)) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibAccessBlocked');
                    } else {
                        var params = {attempts: ++expiredSessionWidget.attempts};
                        MessageService.showMessageByCode('lib#ERR_ATTEMPT_PARTIAL', params, {type: 'ERROR'});
                        if (expiredSessionWidget.attempts >= 3) {
                            logoutExpiredSession();
                        }
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        self.logout();
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    };

    /**
     * @param user -> Object or null.
     *                {
     *                    NAME:  'User Name',
     *                    EMAIL: 'user.email@teknisa.com'
     *                }
     * @param accessProfile -> Array of objects or null. ID is the column NRPERFILACESSO from table PERFILACESSO.
     *                         [
     *                             {
     *                                 ID: '1',
     *                                 IDPERFILACESSOADM: 'S'
     *                             },
     *                             {
     *                                 ID: '2',
     *                                 IDPERFILACESSOADM: 'N'
     *                             }
     *                         ]
     * @param otherSessionValues -> Object or null. PS.: CDFILIALSELECIONADA not exists in session. Use CDFILIAL.
     *                              {
     *                                  CDFILIAL:  '9999',
     *                                  NRPARCNEGOCIO: '1',
     *                                  CDPICTPROD: 'a.a.aa.aaa'
     *                              }
     * @returns Promise
     */
    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var defer = ZHPromise.defer();
        ScreenService.showLoader();
        LibLoginService.updateSession(user, accessProfile, otherSessionValues).then(function(response) {
            ScreenService.hideLoader();
            if(response && response.dataset && response.dataset.sessionUpdated) {
                var sessionUpdated = response.dataset.sessionUpdated[0];
                localStorage.setItem('LOGIN_USER', sessionUpdated);
                defer.resolve();
            } else {
                defer.reject(response);
            }
        }).catch(function(error) {
            ScreenService.hideLoader();
            defer.reject(error);
        });

        return defer.promise;
    };

    this.expiredMenuOnClick = function() {
        MessageService.showMessageByCode('lib#ALE_EXPIRED_MENU', null, { type: 'ERROR' } );
    };

    this.updateTranslation = function() {
        MessageService.confirmMessageByCode('lib#ALE_UPDATE_TRANSLATION', null,
            {
                type: 'CONFIRMATION',
                success: function() {
                    ScreenService.showLoader();
                    LibLoginService.updateTranslation().then(function () {
                        var widget = templateManager.container.getActiveWidget();
                        if(self.checkContainerIsZhNext() && $('iframe').length && widget.sourceUrl) {
                            window.libTranslateDefer = ZHPromise.defer();
                            var frameUrl = widget.sourceUrl.split('?')[0];
                            $('iframe')[0].contentWindow.postMessage({ type: 'updateTranslation' }, frameUrl);
                            window.libTranslateDefer.then(function() {
                                reloadUpdateTranslation();
                            }).catch(function(error) {
                                ScreenService.hideLoader();
                                LibUtilities.showErrorMessage(error);
                            }); 
                        } else {
                            reloadUpdateTranslation();
                        }
                    }).catch(function(error) {
                        ScreenService.hideLoader();
                        LibUtilities.showErrorMessage(error);
                    });
                },
                fail: function () {}
            }
        );
    };

    var reloadUpdateTranslation = function() {
        MessageService.showMessageByCode('lib#SUC_UPDATE_TRANSLATION');
        setTimeout(function() {
            window.location.reload(true);
        }, 7000);
    };

    this.checkContainerIsZhNext = function() {
        if(templateManager.container.widgets && templateManager.container.widgets.length) {
            return templateManager.container.widgets[0].template == 'widget/iframe.html';
        }
        return false;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});